<?php

class CRM_Fromemailpermission_BAO_Fromemailpermission {

  /**
   * Filter from email address restricted by logged in user
   *
   * @param array $options
   *    from email addresses array
   * @param array $userID
   *    logged in User ID
   * @param string $context
   *    [core|api] context to filter array values according to output, due to different array formats
   */
  public static function getFromEmailAddresses(&$options, $userID, $context = 'core') {
    $cacheKey = "CRM_Fromemailpermission_BAO_Fromemailpermission_{$context}_{$userID}";
    $optionsCached = Civi::cache('long')->get($cacheKey);
    if (empty($optionsCached)) {
      $dao = CRM_Core_DAO::executeQuery("SELECT * FROM `civicrm_fromemailpermission`;");

      while ($dao->fetch()) {
        $id = $dao->id;
        $config = unserialize($dao->config);

        // check roles
        if (!empty($config['roles'])) {
          if (!CRM_Core_Permission::checkGroupRole($config['roles'])) {
            self::removeOptionFromArray($options, $id, $context);
            continue;
          }
        }

        // check relationship_type + contact_b_id
        $hasRelationship = [];
        foreach ($config['relationship_type_id'] as $i => $value) {
          if (!empty($config['relationship_type_id'][$i]) && !empty($config['contact_b_id'][$i])) {
            try {
              $params = [
                'contact_id_a' => $userID,
                'contact_id_b' => ['IN' => explode(",", $config['contact_b_id'][$i])],
                'relationship_type_id' => $config['relationship_type_id'][$i],
                'is_active' => 1,
              ];
              $count = civicrm_api3('Relationship', 'getcount', $params);
              $hasRelationship[$i] = !empty($count);
            }
            catch (Exception $e) {
              $hasRelationship[$i] = FALSE;
            }
          }
        }

        $showAddress = TRUE;
        $operator = "AND";
        for ($i = 1; $i <= count($hasRelationship); $i++) {
          if ($i == 1) {
            $showAddress = $hasRelationship[$i];
            $operator = $config['operator'][$i];
          }
          else {
            if ($operator == "OR") {
              $showAddress = $showAddress || $hasRelationship[$i];
            }
            else {
              $showAddress = $showAddress && $hasRelationship[$i];
            }
            $operator = $config['operator'][$i];
          }
        }
        if (!$showAddress) {
          self::removeOptionFromArray($options, $id, $context);
        }
      }

      Civi::cache('long')->set($cacheKey, $options);
    }
    else {
      $options = $optionsCached;
    }
  }

  /**
   * Remove key from options array depending its format
   *
   * @param array $options
   *    from email addresses array
   * @param int $id
   *    option value to delete
   * @param string $context
   *    [core|api] context to filter array values according to output, due to different array formats
   */
  private static function removeOptionFromArray(&$options, $id, $context) {
    switch ($context) {
      case 'core':
        unset($options[$id]);
        break;

      case 'api':
        $tempArray = $options;
        $key = array_search($id, array_column($tempArray, 'value'));
        if ($key) {
          unset($tempArray[$key]);
          $options = array_values($tempArray);
        }
        break;
    }
  }

  /**
   * Generate an array of Domain email addresses.
   *
   * This is a duplicate from CRM_Core_BAO_Email::domainEmails()
   * but ignoring options cache !!!
   *
   * @return array $domainEmails;
   */
  public static function domainEmails() {
    $domainEmails = [];
    $domainFrom = (array) CRM_Core_OptionGroup::values('from_email_address', FALSE, FALSE, FALSE, NULL, 'label', TRUE, TRUE);
    foreach (array_keys($domainFrom) as $k) {
      $domainEmail = $domainFrom[$k];
      $domainEmails[$domainEmail] = htmlspecialchars($domainEmail);
    }
    return $domainEmails;
  }

  /**
   * Build From Email as the combination of all the email ids of the logged in user and
   * the domain email id
   *
   * This is a duplicate from CRM_Core_BAO_Email::getFromEmail()
   * but ignoring options cache
   *
   * @return array
   *   an array of email ids
   */
  public static function getFromEmail() {
    // add all configured FROM email addresses
    $fromEmailValues = self::domainEmails();

    if (!Civi::settings()->get('allow_mail_from_logged_in_contact')) {
      return $fromEmailValues;
    }

    $contactFromEmails = [];
    // add logged in user's active email ids
    $contactID = CRM_Core_Session::getLoggedInContactID();
    if ($contactID) {
      $contactEmails = CRM_Core_BAO_Email::allEmails($contactID);
      $fromDisplayName  = CRM_Core_Session::singleton()->getLoggedInContactDisplayName();

      foreach ($contactEmails as $emailId => $emailVal) {
        $email = trim($emailVal['email']);
        if (!$email || $emailVal['on_hold']) {
          continue;
        }
        $fromEmail = "$fromDisplayName <$email>";
        $fromEmailHtml = htmlspecialchars($fromEmail) . ' ' . $emailVal['locationType'];

        if (!empty($emailVal['is_primary'])) {
          $fromEmailHtml .= ' ' . ts('(preferred)');
        }
        $contactFromEmails[$emailId] = $fromEmailHtml;
      }
    }
    return CRM_Utils_Array::crmArrayMerge($contactFromEmails, $fromEmailValues);
  }

}
